# --- salome initialization
import salome
salome.salome_init()

###
### GEOM component
###

# --- geom Python interface
import GEOM
import math
import random
from salome.geom import geomBuilder
geompy = geomBuilder.New(salome.myStudy)

gg = salome.ImportComponentGUI("GEOM")

specimen_length = 0.160 # laginaren luzera (m)
specimen_width = 0.040 # laginaren zabalera (m)
rebar_diameter = 0.008 # barraren diametroa (m)
rebar_length = 0.150 # barraren luzera (m)
pipe_diameter = 0.010 # tutuaren diametroa (m)
coil_diameter = 0.180 # espiraren diametroa (m)
coil_length = 0.290 # espiraren luzera (m)
coil_turns = 8 # espiraren buelta kopurua
scope_width = 1.000 # azterketa eremua osatzen duen kuboaren neurria (m)

# create origin coordinate
p0 = geompy.MakeVertex(0., 0., 0.)
   
# create auxiliar vector
vectorX = geompy.MakeVectorDXDYDZ(1, 0, 0)
vectorY = geompy.MakeVectorDXDYDZ(0, 1, 0)
vectorZ = geompy.MakeVectorDXDYDZ(0, 0, 1)

# create a rectangle using normal vector (mortar specimen) and translate it to center it
mortar0 = geompy.MakeFaceObjHW(vectorZ, specimen_width/2, specimen_length)
mortar1 = geompy.MakeTranslation(mortar0, specimen_width/4, 0, 0)

# create a rectangle using normal vector (scope) and translate it to center it
scope0 = geompy.MakeFaceObjHW(vectorZ, scope_width/2, scope_width)
scope1 = geompy.MakeTranslation(scope0, scope_width/4, 0, 0)

# create a rectangle using normal vector (rebar) and translate it to center it
rebar0 = geompy.MakeFaceObjHW(vectorZ, rebar_diameter/2, rebar_length)
rebar1 = geompy.MakeTranslation(rebar0, rebar_diameter/4, 0, 0)

# create a disk from a point, a vector and a radius (each of the coil turns) and translate it
coil0 = geompy.MakeDiskPntVecR(p0, vectorZ, pipe_diameter/2)
coil1 = geompy.MakeTranslation(coil0, coil_diameter/2, -coil_length/2, 0)
# translate the given object along the given vector a given number of times (create all the coils turns)
coil2 = geompy.MakeMultiTranslation1D(coil1, vectorY, coil_length/(coil_turns-1), coil_turns)

# create partition (to export to Elmer FEM)
partition1 = geompy.MakePartition([mortar1, scope1, rebar1, coil2], [], [], [], geompy.ShapeType["FACE"], 0, [], 0)

# create groups within the partition for each element
Partition_mortar = geompy.CreateGroup(partition1, geompy.ShapeType["FACE"])
geompy.UnionIDs(Partition_mortar, [2])
Partition_rebar = geompy.CreateGroup(partition1, geompy.ShapeType["FACE"])
geompy.UnionIDs(Partition_rebar, [20])
Partition_scope = geompy.CreateGroup(partition1, geompy.ShapeType["FACE"])
geompy.UnionIDs(Partition_scope, [23])
Partition_coil = geompy.CreateGroup(partition1, geompy.ShapeType["FACE"])
geompy.UnionIDs(Partition_coil, [58, 60, 62, 64, 66, 68, 70, 72])
Partition_boundary = geompy.CreateGroup(partition1, geompy.ShapeType["EDGE"])
geompy.UnionIDs(Partition_boundary, [29, 31, 33])

# load and display elements in the GUI
geompy.addToStudy(p0,"Origin")
geompy.addToStudy(vectorX,"vectorX")
geompy.addToStudy(vectorX,"vectorX")
geompy.addToStudy(vectorY,"vectorY")
geompy.addToStudy(vectorZ,"vectorZ")
geompy.addToStudy(mortar1,"Mortar")
geompy.addToStudy(scope1,"Scope")
geompy.addToStudy(rebar1,"Rebar")
geompy.addToStudy(coil2,"Coil")
geompy.addToStudy(partition1,"Partition")
geompy.addToStudyInFather( partition1, Partition_mortar, 'Partition_mortar' )
geompy.addToStudyInFather( partition1, Partition_rebar, 'Partition_rebar' )
geompy.addToStudyInFather( partition1, Partition_scope, 'Partition_scope' )
geompy.addToStudyInFather( partition1, Partition_coil, 'Partition_coil' )
geompy.addToStudyInFather( partition1, Partition_boundary, 'Partition_boundary' )


# --- update the study object browser
if salome.sg.hasDesktop():
  salome.sg.updateObjBrowser(True)




###
### SMESH component
###

import  SMESH, SALOMEDS
from salome.smesh import smeshBuilder

smesh = smeshBuilder.New(salome.myStudy)

# Assign mesh parameters to body (Partition)
Mesh_1 = smesh.Mesh(partition1)

# Establish NETGEN parameters 
# Sare bakarra
#NETGEN_2D = Mesh_1.Triangle(algo=smeshBuilder.NETGEN_1D2D)
#NETGEN_2D_Parameters = NETGEN_2D.Parameters()
#NETGEN_2D_Parameters.SetSecondOrder( 0 )
#NETGEN_2D_Parameters.SetOptimize( 1 )
#NETGEN_2D_Parameters.SetFineness( 2 )
#NETGEN_2D_Parameters.SetQuadAllowed( 0 )
#NETGEN_2D_Parameters.SetMaxSize( 0.040 )
#NETGEN_2D_Parameters.SetMinSize( 0.001 )

# Altzairua gehiago refinatua
#smesh = smeshBuilder.New(theStudy)
#Mesh_1 = smesh.Mesh(Partition)
NETGEN_2D = Mesh_1.Triangle(algo=smeshBuilder.NETGEN_1D2D)
NETGEN_2D_Parameters = NETGEN_2D.Parameters()
NETGEN_2D_Parameters.SetMaxSize( 0.10 )
NETGEN_2D_Parameters.SetSecondOrder( 0 )
NETGEN_2D_Parameters.SetOptimize( 1 )
NETGEN_2D_Parameters.SetFineness( 2 )
NETGEN_2D_Parameters.SetMinSize( 0.005 )
NETGEN_2D_Parameters.SetQuadAllowed( 0 )
NETGEN_2D_Parameters_1 = smesh.CreateHypothesis('NETGEN_Parameters_2D', 'NETGENEngine')
NETGEN_2D_Parameters_1.SetMaxSize( 0.002 )
NETGEN_2D_Parameters_1.SetSecondOrder( 0 )
NETGEN_2D_Parameters_1.SetOptimize( 1 )
NETGEN_2D_Parameters_1.SetFineness( 3 )
NETGEN_2D_Parameters_1.SetMinSize( 0.002 )
NETGEN_2D_Parameters_1.SetQuadAllowed( 0 )
status = Mesh_1.AddHypothesis(NETGEN_2D,Partition_rebar)
status = Mesh_1.AddHypothesis(NETGEN_2D_Parameters_1,Partition_rebar)
isDone = Mesh_1.Compute()
SubMesh_1 = Mesh_1.GetSubMesh( Partition_rebar, 'SubMesh_1' )

## Set names of Mesh objects
smesh.SetName(NETGEN_2D.GetAlgorithm(), 'NETGEN_2D')
smesh.SetName(NETGEN_2D_Parameters, 'NETGEN 2D Parameters')
smesh.SetName(NETGEN_2D_Parameters_1, 'NETGEN 2D Parameters')
smesh.SetName(SubMesh_1, 'SubMesh_1')
smesh.SetName(Mesh_1.GetMesh(), 'Mesh_1')

# Proceed to mesh the geometry
isDone = Mesh_1.Compute()

# Create mesh groups according to geometry (partition) groups and establish different colours
Mesh_mortar = Mesh_1.GroupOnGeom(Partition_mortar,'Mesh_mortar',SMESH.FACE)
Mesh_mortar.SetColor( SALOMEDS.Color( 1, 1, 1 ))
Mesh_rebar = Mesh_1.GroupOnGeom(Partition_rebar,'Mesh_rebar',SMESH.FACE)
Mesh_rebar.SetColor( SALOMEDS.Color( 0.666667, 0, 0 ))
Mesh_scope = Mesh_1.GroupOnGeom(Partition_scope,'Mesh_scope',SMESH.FACE)
Mesh_scope.SetColor( SALOMEDS.Color( 0.666667, 1, 1 ))
Mesh_coil = Mesh_1.GroupOnGeom(Partition_coil,'Mesh_coil',SMESH.FACE)
Mesh_coil.SetColor( SALOMEDS.Color( 0, 0.333333, 0 ))
Mesh_boundary = Mesh_1.GroupOnGeom(Partition_boundary,'Mesh_boundary',SMESH.EDGE)
Mesh_boundary.SetColor( SALOMEDS.Color( 1, 0.666667, 0 ))

# Export mesh as UNV file format to choosen folder
#Mesh_1.ExportUNV( r'/home/aimar/Dokumentuak/eskola/MAL/2018-2019/alazne arenal/python/axy_rebar.unv' )

## Set names of Mesh objects
smesh.SetName(NETGEN_2D.GetAlgorithm(), 'NETGEN_2D')
smesh.SetName(NETGEN_2D_Parameters, 'NETGEN 2D Parameters')
smesh.SetName(Mesh_mortar, 'Mesh_mortar')
smesh.SetName(Mesh_rebar, 'Mesh_rebar')
smesh.SetName(Mesh_scope, 'Mesh_scope')
smesh.SetName(Mesh_coil, 'Mesh_coil')
smesh.SetName(Mesh_1.GetMesh(), 'Mesh_1')


if salome.sg.hasDesktop():
  salome.sg.updateObjBrowser(1)
